package edu.georgetown.gucs.matcher;

import java.util.concurrent.LinkedBlockingQueue;
import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import edu.georgetown.gucs.matcher.FingerprintMatcher;
import edu.georgetown.gucs.utility.ExceptionHandler;
import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.ObjectBuilder;
import edu.georgetown.gucs.utility.Pair;
import edu.georgetown.gucs.utility.Token;
import edu.georgetown.gucs.utility.Converter;

public class BulkMatcherWorker extends Thread {

    private FingerprintMatcher matcher;
    private String matcherName;
    private int minScore;
    private LinkedBlockingQueue<Pair<Integer,Integer>> matchingQueue;
    private List<Pair<byte[], Token>> fingerprints;
    private List<List<Pair<Token,Token>>> parentResults;
    private List<List<Pair<Token,Token>>> ourResults;
    private Pair<Integer,Integer> poison;
    private AtomicInteger finishedThreads;
	

    /*
     *
     */
    BulkMatcherWorker(String matcherName, int minScore,LinkedBlockingQueue<Pair<Integer,Integer>> toMatch,List<Pair<byte[], Token>> fingerprints, List<List<Pair<Token,Token>>> parentResults, Pair<Integer,Integer> poison, AtomicInteger finishedThreads)   {
	
	this.matcherName = matcherName;
	this.minScore = minScore;
	this.matchingQueue = toMatch;
	this.fingerprints = fingerprints;
	this.parentResults = parentResults;
	this.poison = poison;
	this.finishedThreads = finishedThreads;

	ourResults = new ArrayList<List<Pair<Token,Token>>> (100);

	for (int i = 0; i < 100; i++){
	    this.ourResults.add(i, new ArrayList<Pair<Token,Token>>());
	}
	

    }
    
		      
    @Override
	public void run(){

	Global.monitor.writeToBuffer("BulkMatcherWorker","run","Thead starting" + Thread.currentThread().getName());

	

	// Make the matcher
	this.matcher = (FingerprintMatcher) ObjectBuilder.buildObject(matcherName);

	try {
	    while(true) {
		
		int currentScore = 0;
		
		Pair<Integer, Integer> pair = matchingQueue.take();

		if (pair != poison){
		    Integer first = pair.getA();
		    Integer second = pair.getB();
		
		    for (int x = first + 1; x < second; x++) {
			
			
			byte[] firstFP = fingerprints.get(first).getA();
			byte[] secondFP = fingerprints.get(x).getA();
			
			//System.err.println("Matching: " + Converter.asHex(firstFP) + " to " + Converter.asHex(secondFP));
			
			currentScore = this.matcher.getScore(firstFP,secondFP);
			
			//System.err.println("Thread processing " + first + " " + second + " score: " + currentScore);
			
			if (currentScore >= minScore){
			    
			    // If the score is high enough, create a token and
			    // add it to the slot in the results vector that 
			    // is numbered the same as the score
			    
			    Token firstToken = fingerprints.get(first).getB();
			    Token secondToken = fingerprints.get(x).getB();
			    
			    // Store the results locally in the thread, as the synchronized calls
			    // to pass things to the parent were too slow to do one at a time
			    Pair<Token,Token> matchingFiles = new Pair(firstToken,secondToken);
			    List<Pair<Token,Token>> outerVec = ourResults.get(currentScore);
			    outerVec.add(matchingFiles);
			}
		    }
		}
		else {
		    
		    // When we get the poison, pass all our results back to our parent
		    
		    for (int i = 99; i > 0; i--){
			List<Pair<Token,Token>> ourInnerVec = ourResults.get(i);
			if (ourInnerVec.size() != 0){
			    List<Pair<Token,Token>> parentInnerVec = parentResults.get(i);
			    for (int x = 0; x < ourInnerVec.size(); x++){
				//System.err.println("Thread " + Thread.currentThread().getName() + " adding " + ourInnerVec.get(x).getA() + " " + ourInnerVec.get(x).getB());
				parentInnerVec.add(ourInnerVec.get(x));
			    }

			}
		    }
		    matchingQueue.put(poison);
		    finishedThreads.getAndIncrement();
		    break;
		}
	    }
	}
	catch (InterruptedException e){
	    ExceptionHandler.handleExceptionExit(e);
	}
	Global.monitor.writeToBuffer("BulkMatcherWorker","run","Thead terminating" + Thread.currentThread().getName());
	return;
    }

}