package edu.georgetown.gucs.tokenizers;

import java.util.Collections;
import java.util.List;
import java.util.Vector;
import java.util.regex.Pattern;

import edu.georgetown.gucs.utility.Token;

/**
 * Eliminates tokens containing numbers
 * 
 * @author Clay Shields
 * @author Lindsay Neubauer
 * 
 */
public class RemoveTokensWithNumbersTokenizer extends Tokenizer {

  private static final long serialVersionUID = -5383824450221469920L;
  private static Pattern containsDigitPattern = Pattern.compile(".*\\d.*");

  /**
   * Eliminates tokens containing numbers
   */
  @Override
  public List<Token> tokenize(List<Token> tokens) {
    List<Token> tokenVector = Collections.synchronizedList(new Vector<Token>());
    synchronized (tokenVector) {
      for (Token token : tokens) {
        if (!containsDigitPattern.matcher(token.name()).matches()) {
          tokenVector.add(token);
        }
      }
      return tokenVector;
    }
  }
}
