package edu.georgetown.gucs.utility;

import java.io.File;
import java.util.concurrent.BlockingQueue;
import java.util.Arrays;
import java.util.ArrayList;

/**
 * Thread for processing a file to add to a {@linkplain FileLister}
 * 
 * @author Lindsay Neubauer
 * @author Clay Shields
 */
class FileListerWorker extends Thread {

    private FileLister fileLister;
    private File currentDir;
    private File poison;

    /**
     * Constructor that specifies a file to add to the existing fileLister
     * 
     * @param currentFile
     *          the file to add to the tail of the file queue
     * @param fileLister
     *          the fileLister to add the currentFile to
     */
    FileListerWorker(FileLister theFileLister, File poison) {
	this.fileLister = theFileLister;
	this.poison = poison;
    }

    /**
     * Adds the file to the existing fileLister
     */
    @Override
	public void run() {

	Boolean done = false;
	ArrayList<File> localList = new ArrayList<File>();

	// Get a directory to traverse from the shared queue

	while (!done){

	    this.currentDir = fileLister.getNextDirectory();


	    if (this.currentDir == poison){
		fileLister.addDirectory(poison);
		Global.monitor.writeToBuffer("FileListerWorker", "run", "Finished");
		// The next line doens't matter, but it makes FindBugs happy
		done = true;
		return;
	    }

	    Global.monitor.writeToBuffer("FileListerWorker", "run", "Dir to process: " + currentDir.getAbsolutePath()
					 + " thread_id = " + Thread.currentThread().getName());
		
		
	    try {
		for (File file : Arrays.asList(currentDir.listFiles())) {

		    if (file.isFile() && file.canRead()) {
			// ignore .DS_Store files
			if (file.toString().indexOf(".DS_Store") < 0) {
			    Global.monitor.writeToBuffer("FileListerWorker", "run", "Processed file: " + file);
			    localList.add(file);
			}
		    } else if (file.isDirectory() && file.canRead()) {
			// Ignore .svn directories
			if (file.toString().indexOf(".svn") < 0) {
			    fileLister.addDirectory(file);
			    Global.monitor.writeToBuffer("FileListerWorker", "run", "Added directory: " + file);
			}
		    } else {
			Global.monitor.writeToBuffer("FileListerWorker", "run", "Found odd file: " + file);
		    }
		}
	    }
	    catch (NullPointerException e){
		Global.monitor.printErrorMessage("FileListerWorker","Bad or non-existant directory name: " + currentDir);
		Global.monitor.writeToBuffer("FileListerWorker", "run", "Bad or non-existant directory name: " + currentDir);
		ExceptionHandler.handleExceptionExit(e);
	    }

	    for (File current : localList){
		
		fileLister.addFile(current);
		fileLister.putFile(current);

	    }

	    Global.monitor.writeToBuffer("FileListerWorker", "run", "Finished directory: " + currentDir.getAbsolutePath()
					 + " thread_id = " + Thread.currentThread().getName());

	}
    }

}