package sdtext;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Map.Entry;
import java.util.Set;
import java.util.Vector;

import javax.swing.GroupLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.LayoutStyle;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.SwingWorker;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;

import slider.RangeSlider;
import edu.georgetown.gucs.dictionary.Dictionary;
import edu.georgetown.gucs.dictionary.DictionaryEntry;
import edu.georgetown.gucs.dictionary.ShowDictionaryStatistics;
import edu.georgetown.gucs.dictionary.TrimDictionary;
import edu.georgetown.gucs.utility.Global;
import edu.georgetown.gucs.utility.Token;

/**
 * Provides an interface for receiving action events related to a Dictionary; used with {@linkplain GUI}.
 * 
 * @author Lindsay Neubauer
 * 
 */
class DictionaryActionListener implements ActionListener {
  private TokenizerSelectionHandler tokenizerHandler;
  private JButton dictionaryLink = new JButton();
  private JButton tokensButton = new JButton("Show Dictionary Tokens");
  private JButton statisticsButton = new JButton("Show Dictionary Statistics");
  JButton loadLink = new JButton();
  JButton dictionaryButton = new JButton();
  JLabel minIDFValueLabel = new JLabel("0");
  JLabel maxIDFValueLabel = new JLabel("100");
  RangeSlider idfRangeSlider = new RangeSlider();

  Set<String> idfValues = new HashSet<String>();

  DictionaryActionListener(String createOrTrim) {
    this.tokenizerHandler = new TokenizerSelectionHandler();

    GUI.linkSetup(this.dictionaryLink);
    GUI.linkSetup(this.loadLink);

    this.dictionaryButton.setEnabled(false);
    this.dictionaryButton.setName(createOrTrim);
    this.dictionaryButton.addActionListener(this);
    this.dictionaryButton.setFont(this.minIDFValueLabel.getFont());

    this.idfRangeSlider.setMinimum(0);
    this.idfRangeSlider.setMaximum(100);

    if (createOrTrim.equalsIgnoreCase("create")) {
      this.dictionaryButton.setText("Create Dictionary");
    } else if (createOrTrim.equalsIgnoreCase("trim")) {
      this.dictionaryButton.setText("Trim Dictionary");
    }
  }

  /**
   * Provides the link to an existing dictionary file
   * 
   * @return the JButton containing the link to the dictionary file
   */
  public JButton getLoadDictionaryLink() {
    return this.loadLink;
  }

  /**
   * Sets the link to the existing dictionary file to use
   * 
   * @param loadLink
   *          the JButton containing the link to the dictionary file
   */
  public void setLoadDictionaryLink(JButton loadLink) {
    this.loadLink = loadLink;
  }

  @Override
  public void actionPerformed(ActionEvent actionEvent) {
    Component parent = (Component) actionEvent.getSource();
    if (parent.getName().equals("create")) {
      create(parent);
    } else if (parent.getName().equals("trim")) {
      existing(parent);
    }
  }

  private void create(Component parent) {
    if (this.loadLink.getActionCommand().length() > 0) {
      ((JButton) parent).setEnabled(false);
      if (Double.valueOf(this.minIDFValueLabel.getText()).equals(Double.valueOf(this.maxIDFValueLabel.getText()))) {
        JOptionPane.showConfirmDialog(parent, "The minimum and maximum IDF must not be equal.",
            "Error: Normalized IDF Range", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
      } else {
        this.dictionaryLink.setText("<HTML>Creating Dictionary...</HTML>");
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileFilter(new FileNameExtensionFilter("XML Files", "xml"));
        fileChooser.setDialogTitle("Save as XML Dictionary File");
        int returnVal = fileChooser.showSaveDialog(parent);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
          try {
            Dictionary dictionary = new Dictionary();
            dictionary.makeDictionary(this.tokenizerHandler.getTokenizers(), this.loadLink.getActionCommand(),
                Double.valueOf(this.minIDFValueLabel.getText()), Double.valueOf(this.maxIDFValueLabel.getText()),
                fileChooser.getSelectedFile().getAbsolutePath());
            GUI.createLink(dictionary.getDictionaryFilename(), this.dictionaryLink, false);
          } catch (RuntimeException exception) {
            this.dictionaryLink.setText("");
            Global.monitor.printErrorMessage("DictionaryActionListener", "Exception: " + exception.getMessage());
            Global.monitor.writeToBuffer("DictionaryActionListener", "dictionaryButton.actionPerformed.create",
                exception.getMessage());
            Global.monitor.writeToBuffer("DictionaryActionListener", "dictionaryButton.actionPerformed.create",
                Arrays.toString(exception.getStackTrace()));
            JOptionPane.showConfirmDialog(parent, "Exception: " + exception.getMessage(), "Error: Dictionary",
                JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
          }
        } else {
          this.dictionaryLink.setText("");
        }
      }
      ((JButton) parent).setEnabled(true);
    } else {
      JOptionPane.showConfirmDialog(parent, "Must set the dictionary directory before creating a dictionary",
          "Error: Create Dictionary", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
    }
  }

  private void existing(Component parent) {
    if (this.loadLink.getActionCommand().length() > 0) {
      ((JButton) parent).setEnabled(false);
      if (Double.valueOf(this.minIDFValueLabel.getText()).equals(Double.valueOf(this.maxIDFValueLabel.getText()))) {
        JOptionPane.showConfirmDialog(parent, "The minimum and maximum IDF must not be equal.",
            "Error: Normalized IDF Range", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
      } else {
        this.dictionaryLink.setText("<HTML>Creating Dictionary...</HTML>");
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setFileFilter(new FileNameExtensionFilter("XML Files", "xml"));
        fileChooser.setDialogTitle("Save as XML Dictionary File");
        int returnVal = fileChooser.showSaveDialog(parent);
        if (returnVal == JFileChooser.APPROVE_OPTION && this.loadLink.getActionCommand().length() > 0) {
          try {
            TrimDictionary trimDict = new TrimDictionary(Double.valueOf(this.minIDFValueLabel.getText()),
                Double.valueOf(this.maxIDFValueLabel.getText()), this.loadLink.getActionCommand());
            trimDict.trim(fileChooser.getSelectedFile().getAbsolutePath());
            GUI.createLink(fileChooser.getSelectedFile().getAbsolutePath(), this.dictionaryLink, false);
          } catch (RuntimeException exception) {
            this.dictionaryLink.setText("");
            Global.monitor.printErrorMessage("DictionaryActionListener", "Exception: " + exception.getMessage());
            Global.monitor.writeToBuffer("DictionaryActionListener", "dictionaryButton.actionPerformed.trim",
                exception.getMessage());
            Global.monitor.writeToBuffer("DictionaryActionListener", "dictionaryButton.actionPerformed.trim",
                Arrays.toString(exception.getStackTrace()));
            JOptionPane.showConfirmDialog(parent, "Exception: " + exception.getMessage(), "Error: Dictionary",
                JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
          }
        } else {
          this.dictionaryLink.setText("");
        }
      }
      ((JButton) parent).setEnabled(true);
    } else {
      JOptionPane.showConfirmDialog(parent, "Must load the existing dictionary before trimming",
          "Error: Trim Dictionary", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
    }
  }

  /**
   * Creates the panel that contains all necessary components for loading an existing dictionary
   * 
   * @param layout
   *          the groupLayout to use for loading an existing dictionary
   */
  protected void createLoadDictionaryPanel(GroupLayout layout) {
    JPanel dictionaryPanel = createLoadPanel("Existing Dictionary", JFileChooser.FILES_ONLY, Global.XML_ONLY);
    final JTextArea statisticsTextArea = new JTextArea("");
    statisticsTextArea.setFont(Global.unifontFont14);
    statisticsTextArea.setEditable(false);
    JScrollPane statisticsScrollPane = new JScrollPane(statisticsTextArea,
        ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    statisticsScrollPane.setMinimumSize(new Dimension(50, 125));
    this.statisticsButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(final ActionEvent actionEvent) {
        if (DictionaryActionListener.this.loadLink.getActionCommand().length() > 0) {
          ((JButton) actionEvent.getSource()).setEnabled(false);
          statisticsTextArea.setText("Loading Dictionary Statistics....");
          SwingWorker<String, String> worker = new SwingWorker<String, String>() {
            @Override
            protected String doInBackground() {
              String statistics = ShowDictionaryStatistics.getStatistics(DictionaryActionListener.this.loadLink
                  .getActionCommand());
              statisticsTextArea.setText(statistics);
              ((JButton) actionEvent.getSource()).setEnabled(true);
              return "";
            }
          };
          worker.execute();
        } else {
          statisticsTextArea.setText("");
          JOptionPane.showConfirmDialog((Component) actionEvent.getSource(), "Must load the existing dictionary",
              "Error: Show Dictionary Statistics", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
        }
      }
    });
    JPanel statisticsPanel = GUI.createSubPanel(
        Arrays.asList(new JComponent[] { this.statisticsButton, statisticsScrollPane }), "Show Dictionary Statistics",
        true);

    final DefaultTableModel tokenTableModel = new DefaultTableModel(new Vector<String[]>(), new Vector<String>(
        Arrays.asList(new String[] { "Token", "Frequency", "IDF", "Normalized IDF" })));
    JTable tokensTable = new JTable(tokenTableModel);
    tokensTable.setEnabled(false);
    tokensTable.setFillsViewportHeight(true);
    tokensTable.setAutoCreateRowSorter(true);
    JScrollPane tokensScrollPane = new JScrollPane(tokensTable, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    tokensScrollPane.setPreferredSize(new Dimension(50, 125));
    this.tokensButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(final ActionEvent actionEvent) {
        if (DictionaryActionListener.this.loadLink.getActionCommand().length() > 0) {
          tokenTableModel.setRowCount(0);
          ((JButton) actionEvent.getSource()).setEnabled(false);
          SwingWorker<String, String> worker = new SwingWorker<String, String>() {
            @Override
            protected String doInBackground() {
              Dictionary dictionary = new Dictionary(DictionaryActionListener.this.loadLink.getActionCommand());
              for (Entry<Token, DictionaryEntry> entry : dictionary) {
                tokenTableModel.addRow(new String[] { entry.getKey().name(),
                    String.valueOf(entry.getValue().getFrequencyCount()), String.valueOf(entry.getValue().getIDF()),
                    String.valueOf(dictionary.getNormalizedIDF(entry.getValue())) });
              }
              ((JButton) actionEvent.getSource()).setEnabled(true);
              return "";
            }
          };
          worker.execute();
        } else {
          tokenTableModel.setRowCount(0);
          JOptionPane.showConfirmDialog((Component) actionEvent.getSource(), "Must load the existing dictionary",
              "Error: Show Dictionary Tokens", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
        }
      }
    });
    JPanel tokensPanel = GUI.createSubPanel(Arrays.asList(new JComponent[] { this.tokensButton, tokensScrollPane }),
        "Show Dictionary Tokens", true);
    JPanel trimPanel = createDictionaryPanel("Trim Dictionary", true);

    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(dictionaryPanel).addComponent(statisticsPanel)
        .addComponent(tokensPanel).addComponent(trimPanel));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(dictionaryPanel)
        .addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(statisticsPanel)
        .addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(tokensPanel)
        .addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(trimPanel));
  }

  /**
   * Creates the panel that contains all necessary components for dictionary creation
   * 
   * @param layout
   *          the groupLayout to use for the new dictionary creation panel
   */
  protected void createNewDictionaryPanel(GroupLayout layout) {
    JPanel tokenizersPanel = new JPanel();
    this.tokenizerHandler.createPanel(GUI.createGroupLayout(tokenizersPanel), false, false);
    JPanel directoryPanel = createLoadPanel("Directory", JFileChooser.DIRECTORIES_ONLY, Global.ALL_FILES);
    JPanel dictionaryPanel = createDictionaryPanel("Normalized IDF Range", false);

    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(tokenizersPanel).addComponent(directoryPanel)
        .addComponent(dictionaryPanel));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(tokenizersPanel).addComponent(directoryPanel)
        .addPreferredGap(LayoutStyle.ComponentPlacement.UNRELATED).addComponent(dictionaryPanel));
  }

  /**
   * Creates the panel that contains all necessary components for setting the IDF ranges
   * 
   * @param idfRangeLabel
   *          the JLabel displaying the min and max IDF section heading
   * @param includeIDFButton
   *          the boolean value indicating whether to include a button for adding and deleting multiple sets of IDF
   *          ranges
   * 
   * @return the JPanel containing all the necessary IDF selection components
   */
  protected JPanel createIDFPanel(JLabel idfRangeLabel, boolean includeIDFButton) {
    JPanel idfRangePanel = new JPanel();
    GroupLayout layout = GUI.createGroupLayout(idfRangePanel);

    JPanel minMaxIDFPanel = new JPanel(new GridLayout(2, 0));
    minMaxIDFPanel.add(new JLabel("Minimum IDF: "));
    minMaxIDFPanel.add(this.minIDFValueLabel);
    minMaxIDFPanel.add(new JLabel("Maximum IDF: "));
    minMaxIDFPanel.add(this.maxIDFValueLabel);

    Hashtable<Integer, JLabel> idfLabelTable = new Hashtable<Integer, JLabel>();
    for (int i = 0; i <= 100; i += 10) {
      idfLabelTable.put(Integer.valueOf(i), new JLabel(String.valueOf(i / 100.0)));
    }
    this.idfRangeSlider.setMajorTickSpacing(10);
    this.idfRangeSlider.setMinorTickSpacing(5);
    this.idfRangeSlider.setLabelTable(idfLabelTable);
    this.idfRangeSlider.setPaintTicks(true);
    this.idfRangeSlider.setPaintLabels(true);
    this.idfRangeSlider.addChangeListener(new ChangeListener() {
      @Override
      public void stateChanged(ChangeEvent e) {
        RangeSlider slider = (RangeSlider) e.getSource();
        DictionaryActionListener.this.minIDFValueLabel.setText(String.valueOf(slider.getValue() / 100.0));
        DictionaryActionListener.this.maxIDFValueLabel.setText(String.valueOf(slider.getUpperValue() / 100.0));
      }
    });
    if (this.dictionaryButton.getText().equals("Trim Dictionary")) {
      this.idfRangeSlider.setValue(30);
      this.idfRangeSlider.setUpperValue(70);
    } else {
      this.idfRangeSlider.setValue(0);
      this.idfRangeSlider.setUpperValue(100);
    }
    this.minIDFValueLabel.setText(String.valueOf(this.idfRangeSlider.getValue() / 100.0));
    this.maxIDFValueLabel.setText(String.valueOf(this.idfRangeSlider.getUpperValue() / 100.0));

    JButton addButton = new JButton("Add IDF Range");
    final JButton clearButton = new JButton("Delete IDF Range");
    addButton.setVisible(includeIDFButton);
    clearButton.setVisible(includeIDFButton);
    clearButton.setEnabled(false);

    final SelectableTableModel idfTableModel = new SelectableTableModel(new Vector<String>(), new Vector<String>(
        Arrays.asList(new String[] { "Minimum IDFs", "Maximum IDFs" })));
    final JTable IDFTable = new JTable(idfTableModel);
    JScrollPane tokenizerTableScrollPane = new JScrollPane(IDFTable, ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
        ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    tokenizerTableScrollPane.setVisible(includeIDFButton);
    tokenizerTableScrollPane.setPreferredSize(new Dimension(50, 125));
    addButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent actionEvent) {
        if (DictionaryActionListener.this.idfValues.add(DictionaryActionListener.this.idfRangeSlider.getValue() + ","
            + DictionaryActionListener.this.idfRangeSlider.getUpperValue())) {
          idfTableModel.addRow(new Double[] { DictionaryActionListener.this.idfRangeSlider.getValue() / 100.0,
              DictionaryActionListener.this.idfRangeSlider.getUpperValue() / 100.0 });
          DictionaryActionListener.this.idfRangeSlider.setValue(0);
          DictionaryActionListener.this.idfRangeSlider.setUpperValue(100);
          DictionaryActionListener.this.minIDFValueLabel.setText("0");
          DictionaryActionListener.this.maxIDFValueLabel.setText("100");
          clearButton.setEnabled(true);
        }
      }
    });
    clearButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent arg0) {
        int[] rows = IDFTable.getSelectedRows();
        for (int i = 0; i < rows.length; i++) {
          Double min = (Double) IDFTable.getValueAt(rows[i] - i, 0) * 100;
          Double max = (Double) IDFTable.getValueAt(rows[i] - i, 1) * 100;
          DictionaryActionListener.this.idfValues.remove(min.toString().replace(".0", "") + ","
              + max.toString().replace(".0", ""));
          idfTableModel.removeRow(rows[i] - i);
        }
        if (DictionaryActionListener.this.idfValues.size() == 0) {
          clearButton.setEnabled(false);
        }
      }
    });

    layout.setHorizontalGroup(layout
        .createParallelGroup()
        .addComponent(idfRangeLabel)
        .addGroup(
            layout.createSequentialGroup()
                .addPreferredGap(idfRangeLabel, minMaxIDFPanel, LayoutStyle.ComponentPlacement.INDENT)
                .addComponent(minMaxIDFPanel).addComponent(this.idfRangeSlider))
        .addGroup(layout.createSequentialGroup().addComponent(addButton).addComponent(clearButton))
        .addComponent(tokenizerTableScrollPane));
    layout.setVerticalGroup(layout
        .createSequentialGroup()
        .addComponent(idfRangeLabel)
        .addGroup(layout.createParallelGroup().addComponent(minMaxIDFPanel).addComponent(this.idfRangeSlider))
        .addGroup(
            layout.createParallelGroup(GroupLayout.Alignment.CENTER).addComponent(addButton).addComponent(clearButton))
        .addComponent(tokenizerTableScrollPane));
    layout.linkSize(SwingConstants.VERTICAL, minMaxIDFPanel, this.idfRangeSlider);

    return idfRangePanel;
  }

  /**
   * Provides the set of all min and max IDF value combinations
   * 
   * @return the set of strings, each containing a min and max IDF value
   */
  protected Set<String> getIDFValues() {
    return this.idfValues;
  }

  private JPanel createDictionaryPanel(String labelText, boolean twoHeading) {
    JPanel dictionaryPanel = new JPanel();
    GroupLayout layout = GUI.createGroupLayout(dictionaryPanel);

    JLabel dictionaryLabel = new JLabel(labelText);
    dictionaryLabel.setFont(Global.boldUnifontFont);
    JPanel idfPanel = (twoHeading) ? createIDFPanel(new JLabel("Normalized IDF Range"), false) : createIDFPanel(
        new JLabel(), false);

    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(dictionaryLabel).addComponent(idfPanel)
        .addComponent(this.dictionaryLink).addComponent(this.dictionaryButton));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(dictionaryLabel).addComponent(idfPanel)
        .addComponent(this.dictionaryLink).addComponent(this.dictionaryButton));
    return dictionaryPanel;
  }

  /**
   * Provides a panel that allows a user to load a file or directory
   * 
   * @param panelLabel
   *          string name of panel to create
   * @param fileSelectionMode
   *          int instruction on which files to display (JFileChooser.FILES_ONLY or JFileChooser.DIRECTORIES_ONLY)
   * @param showAll
   *          boolean value indicating which files to display (Global.ALL_FILES or Global.XML_ONLY)
   * 
   * @return the JPanel containing the label, link, and bottom that allows user to load file/directory
   */
  private JPanel createLoadPanel(final String panelLabel, final int fileSelectionMode, final boolean showAll) {
    JLabel label = new JLabel(panelLabel);
    label.setFont(Global.boldUnifontFont);
    final JButton directoryButton = new JButton("Select " + panelLabel);
    directoryButton.setFont((new JLabel()).getFont());
    directoryButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent actionEvent) {
        directoryButton.setEnabled(false);
        DictionaryActionListener.this.loadLink.setText("<HTML>Loading " + panelLabel + "...</HTML>");
        JFileChooser chooser = new JFileChooser();
        if (!showAll) {
          chooser.setFileFilter(new FileNameExtensionFilter("XML Files", "xml"));
        }
        chooser.setFileSelectionMode(fileSelectionMode);
        chooser.setDialogTitle("Choose a " + panelLabel);
        int returnVal = chooser.showOpenDialog((Component) actionEvent.getSource());
        if (returnVal == JFileChooser.APPROVE_OPTION) {
          if (panelLabel.contains("Dictionary")
              && !GUI.xmlFileTypeCheck("dictionary", chooser.getSelectedFile().getAbsolutePath())) {
            DictionaryActionListener.this.loadLink.setText("");
            DictionaryActionListener.this.loadLink.setActionCommand("");
            JOptionPane.showConfirmDialog((Component) actionEvent.getSource(), "Must load existing dictionary",
                "Error: Dictionary", JOptionPane.CANCEL_OPTION, JOptionPane.ERROR_MESSAGE);
          } else {
            DictionaryActionListener.this.loadLink.setActionCommand(chooser.getSelectedFile().getAbsolutePath());
            GUI.createLink(chooser.getSelectedFile().getAbsolutePath(), DictionaryActionListener.this.loadLink, false);
            if (DictionaryActionListener.this.dictionaryButton != null) {
              DictionaryActionListener.this.dictionaryButton.setEnabled(true);
            }
          }
        } else {
          DictionaryActionListener.this.loadLink.setText("");
          DictionaryActionListener.this.loadLink.setActionCommand("");
        }
        directoryButton.setEnabled(true);
      }
    });

    JPanel panel = new JPanel();
    GroupLayout layout = GUI.createGroupLayout(panel);
    layout.setHorizontalGroup(layout.createParallelGroup().addComponent(label).addComponent(this.loadLink)
        .addComponent(directoryButton));
    layout.setVerticalGroup(layout.createSequentialGroup().addComponent(label).addComponent(this.loadLink)
        .addComponent(directoryButton).addPreferredGap(label, this.loadLink, LayoutStyle.ComponentPlacement.INDENT)
        .addPreferredGap(label, directoryButton, LayoutStyle.ComponentPlacement.INDENT));
    return panel;
  }

}